#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module MonitorHelper
  def monitor_cast_num(count)
    cast_num count, ''
  end

  def call_hardwareinfo
    device();
  end

  def monitor_name(monitor)
    sensor_name = monitor.sensor_name
    group_name = monitor.group_name
    instance = monitor.instance
    name = sensor_name

    if instance.present?
      if group_name.present?
        name = (instance + ' (' + name + ' - ' + group_name + ')' )
      else
        name = (instance + ' (' + name + ')' )
      end
    else
      name = (name + ' - ' + group_name) if group_name.present?
    end

    name
  end

  def cast_monitor(count)
    if count > 1e12
      count = (count.to_f / 1e12).round(3)
      count = count.to_s + ' T'
    elsif count > 1e9
      count = (count.to_f / 1e9).round(3)
      count = count.to_s + ' G'
    elsif count > 1e6
      count = (count.to_f / 1e6).round(3)
      count = count.to_s + ' M'
    elsif count > 1e3
      count = (count.to_f / 1e3).round(3)
      count = count.to_s + ' K'
    else
      count = count.round(3).to_s
    end
    count
  end

  #
  # Return the namespace for a given segment
  #
  # == Parameters:
  # segment::
  #   Segment complete source with the data associated
  #
  # == Returns:
  # String with the namespace for the given segment
  #
  def segment_namespace_name(segment)
    uuid = segment.datasource.match(/^rb_\w+_(?<uuid>.+)/)[:uuid]
    current_user.data_sources.find{ |d| d.uuid.eql?(uuid) }.name
  rescue
    'Default'
  end

  #
  # Return the source for a given segment
  #
  # == Parameters:
  # segment::
  #   Segment complete source with the data associated
  #
  # == Returns:
  # String with the source for the given segment
  #
  def segment_source_name(segment)
    match = segment.datasource.match(/^rb_(?<topic>[^_]+)/)
    return 'N/A' unless match
    name = match[:topic]
    if name == 'ips'
      name.upcase
    elsif name == 'loc'
      'Location'
    else
      name.capitalize
    end
  end

  #
  # Return four rows with the identifier, datasource, namespace, type and
  # interval.
  #
  # == Parameters:
  # task::
  #   Task object with the data associated
  #
  # == Returns:
  # [HTML Safe] String
  #
  def task_columns(task)
    data = []
    if task.id.start_with? 'kill'
      type = 'kill'
      interval = task.payload['interval'] || '-'
    elsif task.payload['spec'] && task.payload['spec']['ioConfig']
      type = task.payload['spec']['ioConfig']['type']
      interval =
        task.payload['spec']['ioConfig']['firehose'].try(:[], 'interval') || '-'
    else
      type = '-'
      interval = '-'
    end
    # Get the partition and replica
    partition, replica = partition_and_replica(task.id)

    # Get the datasource
    datasource = task.payload['dataSource']
    m = /\Arb_(?<topic>([a-z])+(_[a-z]+){0,1})(?<namespace>_[a-zA-Z0-9-]*)*\z/
        .match(datasource)
    namespace = m[:namespace] ? m[:namespace][1..m[:namespace].size] : 'default'
    data += [['Topic', m[:topic]], ['Namespace', namespace], ['Type', type],
             ['Interval', gsub_interval(interval)], ['Partition', partition],
             ['Replica', replica]]
    # Join and return
    data.map { |name, el| content_tag(:td, el, data: { header: name }) }.join.html_safe
  end

  # Get the partition and replica from a task
  #
  # @param id [String] Identifier of the task
  # @return [Array] The first element is the partition and the second the
  #    replica
  #
  def partition_and_replica(id)
    return ['-', '-'] if id.start_with? 'kill'
    el = id.split('.000Z').last
    m = /\A_(?<partition>[0-9]+)_(?<replica>[0-9]+)\z/.match(el)
    m ? [m[:partition], m[:replica]] : ['-', '-']
  end

  # Remove timezone from interval string
  #
  # @param interval [String] interval to remove timezone
  # @return [String] interval without timezone
  #
  def gsub_interval(interval)
    return if interval == '-'
    interval = interval.gsub(':00.000Z', '')
    interval = interval.gsub('.000Z', '')
    interval = interval.gsub('T', ' ')
    interval.gsub('/', ' / ')
  end

  # 
  # Druid indexer checks
  #
  def check_indexer_nodes(available_indexers, zk_indexers)
    available = available_indexers.count.to_i
    expected  = zk_indexers.count.to_i

    return :error    if expected.zero?
    return :critical if available < expected
    return :warning  if available > expected
    :ok
  end

  def get_message_indexer_status(status)
    case status
    when :error    then "No indexer nodes registered in ZooKeeper"
    when :critical then "Fewer available indexers than expected"
    when :warning  then "More available indexers than expected"
    when :ok       then "Indexer count matches ZooKeeper"
    end
  end

  def druid_indexer_get_status(available_indexers, zk_indexers)
    status = check_indexer_nodes(available_indexers, zk_indexers)
    icon   = get_type_icon_label(status.to_s)
    title  = get_message_indexer_status(status)
    "<span class=\"#{status} bold\" data-name=\"#{status}\" title=\"#{title}\">#{status.to_s.upcase} #{icon}</span>".html_safe
  end

  #
  # Get Errors Druid Historical
  #

  # Check if Druid Historicals are missing
  def druid_historical_check_number_historicals(hosts:, elements:, type: 'warning', current: {})
    hosts_down = []
    active_historicals = hosts.length
    hosts.each do |m|
      index_hist = elements.index { |h| h.host.split('.').first == m}
      index_hist.nil? ? hosts_down.append(m) : active_historicals-=1
    end
    return current if active_historicals.zero?
    type = 'error' if active_historicals == hosts.length
    hosts_down.each { |h| current["#{h}"] = type }
    current
  end

  # Check Used percent less than threshold
  def druid_check_size_error(hosts:, elements:, element_type: 'historicals', threshold: 75, type: 'warning', current: {})
    elements_used = get_cluster_element(elements, element_type) if get_cluster_element(elements, element_type)
    elements_used.each_with_index do |el, index|
      next if el.used_percent < threshold
      if element_type == 'tiers'
        current["#{el.name}"] = type
      elsif element_type == 'workers'
        current["#{el.host}"] = type
      else
        current["#{el}"] = type
      end
    end
    current
  end

  # Check Max size is not zero
  def druid_historical_check_empty_error(hosts:, elements:, element_type: 'historicals', type: 'warning', current: {})
    elements_used = get_cluster_element(elements, element_type) if get_cluster_element(elements, element_type)
    if element_type == 'historicals' and elements_used.length != hosts.length
      hosts.each do |m|
        index_hist = elements_used.index { |h| h.host.split('.').first == m}
        current["#{m}"] = type if index_hist.nil?
      end
    else
      elements_used.each_with_index do |el, index|
        next if el.max_size != 0
        current[element_type == 'tiers' ? "#{el.name}" : "#{el.host}"] = type
      end
    end
    current
  end

  # Check Tier is not empty
  def druid_historical_check_empty_tier_error(hosts:, elements:, element_type: 'historicals', type: 'warning', current: {})
    tiers_empty = []
    elements_used = get_cluster_element(elements, element_type) if get_cluster_element(elements, element_type)
    elements_used.each_with_index do |el, index|
      nodes_in_historical = 0
      el.nodes.each_with_index do |el_node, index|
        nodes_in_historical += 1 if el_node.type.to_s.downcase.include?("historical")
      end
      next if nodes_in_historical != 0
      tiers_empty.append(el)
    end
    type = 'error' if tiers_empty.length == elements_used.length
    tiers_empty.each { |t| current["#{t.name}"] = type }
    current
  end

  def get_cluster_element(cluster, element)
    case element
    when 'historicals'
      cluster.historicals.sort_by(&:tier)
    when 'tiers'
      cluster.tiers.sort_by(&:name)
    when 'workers'
      cluster.workers
    else
      nil
    end
  end

  def druid_historical_get_all_errors(hosts, cluster)
    errors = {
      missing_historicals: {},
      limit_use_hist: {},
      empty_size_hist: {},
      limit_use_tier: {},
      empty_size_tier: {},
      tier_empty: {}
    }
    errors_level = {
      warning: 'warning',
      error: 'error'
    }

    # Check if all registered Druid Historicals are active
    begin
      errors[:missing_historicals] = druid_historical_check_number_historicals(hosts: hosts, elements: cluster.historicals)
    rescue
      errors[:missing_historicals] = {ERROR: 'druid_historical_check_number_historicals'}
    end

    # Check limits of Druid Historical and Tiers
    begin         
      errors[:limit_use_hist] = druid_check_size_error(hosts: hosts, elements: cluster, threshold: 80)
      errors[:limit_use_tier] = druid_check_size_error(hosts: hosts, elements: cluster, element_type: 'tiers', threshold: 80, type: errors_level[:error])
    rescue
      errors[:limit_use_hist] = {ERROR: 'druid_check_size_error'}
    end

    # Check not zero size of Druid Historical and Tiers
    begin         
      errors[:empty_size_hist] = druid_historical_check_empty_error(hosts: hosts, elements: cluster)
      errors[:empty_size_tier] = druid_historical_check_empty_error(hosts: hosts, elements: cluster, element_type: 'tiers')
    rescue
      errors[:empty_size_hist] = {ERROR: 'druid_historical_check_empty_error'}
    end

    # Check if tier has active nodes
    begin
      errors[:tier_empty] = druid_historical_check_empty_tier_error(hosts: hosts, elements: cluster, element_type: 'tiers')
    rescue
      errors[:tier_empty] = {ERROR: 'druid_historical_check_empty_tier_error'}
    end
    errors
  end

  def check_all_hosts_used_in_historical_nodes(tiers, hosts)
    content = []
    hosts.each_with_index do |host, index|
      content.append(check_host_used_in_historical_nodes(tiers, host))
    end
    content
  end
  
  def check_host_used_in_historical_nodes(tiers, host)
    info = {}
    tiers.any? do |tier|
      tier.nodes.any? do |node|
        if node.type == :historical && node.host.split('.').first == host
          info[:host] = host
          info[:tier] = tier.name
        end
      end
    end
    return info
  end

  def get_class_type_errors_css(type)
    case type
    when 'historicals'
      '.cluster-historical-nodes'
    when 'tiers'
      '.cluster-tier-nodes'
    else
      'all'
    end
  end

  def get_type_icon_label(label)
    icon_class = case label.downcase
    when 'ok'
      'fa fa-check icon-green'
    when 'error'
      'fa fa-times icon-red'
    when "warning"
      "fa fa-exclamation-triangle icon-yellow"
    else
      "fa fa-question-circle icon-gray"
    end
    icon_content = content_tag(:i, "", class: icon_class)
  end

  def get_message_status(error_level)
    error_level = case error_level.downcase
    when 'ok'
      'System is operating normally'
    when 'error'
      'Failed System'
    when "warning"
      "Abnormal behaviour in the System"
    else
      "No data of error"
    end
  end

  def create_status_messsage_element(content, error_level)
    content_tag :span, "#{content.upcase} #{get_type_icon_label(error_level)}".html_safe,
            class: "#{error_level} bold",
            data: { name: error_level },
            title: "#{get_message_status(error_level)}"
  end

  def check_errors(errors, host = '')
    if !host.length.zero?
      errors = errors.transform_values do |v|
        v.is_a?(Hash) ? v.select { |k, _| k == host } : v
      end
    end
    if errors.values.any? { |v| !v.nil? && !v.empty? }     
      error = errors.sum { |_, v| v.values.count("error") }
      if error > 0
        return "error"
      end

      warning = errors.sum { |_, v| v.values.count("warning") }
      if warning > 0
        return "warning"
      end
    end
    return "ok"
  end

  def druid_historical_get_summary_status(content, errors)
    error_level = check_errors(errors)
    if content.length.zero?
      content << error_level
    end
    create_status_messsage_element(content, error_level)
  end

  def tops_values(val)
    val.is_a?(Numeric) ? cast_monitor(val) : 0.0
  end

  def module_controller_name_monitor
    'monitor'
  end

  def get_oid_from_operation(operation)

     if operation.kind_of?(Array)
       oper = ActiveSupport::JSON.decode(operation.first.to_json)
     else
       oper = ActiveSupport::JSON.decode(operation.to_json)
     end


     return ""

      # if oper.key? "system"
      #   return oper["system"]
      # else
      #   return ""
      # end
  end

  def druid_indexing_get_all_errors(hosts, cluster)
    errors = {
      limit_use: {}
    }

    # Check limits of Druid indexer
    begin         
      errors[:limit_use] = druid_check_size_error(hosts: hosts, elements: cluster, element_type: 'workers', threshold: 90)
    rescue
      errors[:limit_use] = {ERROR: 'druid_check_size_error'}
    end

    errors
  end

  ## ======================================================================
  # => DRUID QUERY LOGGING METHODS
  ## ======================================================================

  def parse_data_source(log)
    data_source = log['dataSource']

    case data_source
    when String
      data_source # Return the string directly
    when Hash
      data_source.dig('dataSources', 0) || 'N/A' # Extract first data source
    else
      'N/A'
    end
  end

  def cache_key_for_druid_query_logging(field, user_id)
    "rb-webui:druid_query_logging:#{field}:user:#{user_id}"
  end
end
