#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class UpdateInventoryDevicesJob < Struct.new(:user_id, :verbose)
      include Redborder::Jobs::JobHelper
      attr_accessor :content

      def perform
        content = "Node executing this job: #{`hostname`.chomp}\n\n"

        begin
          manager_role = ::Chef::Role.manager

          if manager_role.override_attributes['redborder']['inventory_devices']['sources']['traffic'] == "1"
            Delayed::Worker.logger.info "Getting inventory device from source Traffic"
            content += "Getting inventory device from source Traffic\n"
            traffic_task = InventoryDeviceFlowDiscovery.new
            content += traffic_task.execute.to_s
            check_success = $?.success?
            raise content unless check_success
          else
            Delayed::Worker.logger.info "Skipping Traffic: no source Traffic selected to update inventory devices"
            content += "Skipping Traffic: no source Traffic selected to update inventory devices\n"
          end

          if manager_role.override_attributes['redborder']['inventory_devices']['sources']['glpi'] == "1"
            glpi_jobs_count = Delayed::Job.where("handler LIKE ?", "%!ruby/struct:#{self.class.name}%").count
            if glpi_jobs_count <= 1
              unless DelayedJob.glpi_inventory_job?
                Delayed::Worker.logger.info "Starting GLPI job"
                content += "Starting GLPI job\n"
                Delayed::Job.enqueue(Redborder::Jobs::GlpiInventoryJob.new(user_id, true), priority: 10)
              else
                 Delayed::Worker.logger.info "No need to start GLPI job as it's already in the queue"
                content += "No need sto start GLPI job as its already in the queue\n"
              end
            else
               Delayed::Worker.logger.info "GLPI inventory job is already queue for run (Check this job settings from the GLPI integration section)"
              content += "GLPI inventory job is already queue for run (Check this job settings from the GLPI integration section)\n"
            end
          else
            Delayed::Worker.logger.info "Skipping GLPI: no source GLPI selected to update inventory devices"
            content += "\nSkipping GLPI: no source GLPI selected to update inventory devices\n"
          end

          is_job_ready = true
        rescue => e
          content += "Error: #{e.message}\n"
          e.backtrace.each { |line| content += "#{line}\n" }
          Delayed::Worker.logger.info "Error in UpdateInventoryDevicesJob: #{e.message}"
          logit "#{e.class}: #{e.message}" if verbose
          raise e
        ensure
          StoredDelayedJob.create(job: "UpdateInventoryDevicesJob", user_id: user_id, content: content)
          if is_job_ready
            attempts = Delayed::Job.where(handler: self.to_yaml).pluck(:attempts).first || 0
            next_run_time = reschedule_at(Time.now, attempts)
            if next_run_time
              Delayed::Job.enqueue(self.class.new(user_id, verbose), run_at: next_run_time)
              Delayed::Worker.logger.info "Job UpdateInventoryDevicesJob rescheduled to #{next_run_time}"
            else
              Delayed::Worker.logger.info "Job UpdateInventoryDevicesJob will not be rescheduled"
            end
          else
             Delayed::Worker.logger.info "Job UpdateInventoryDevicesJob failed and will not be rescheduled"
          end
        end
      end

      def reschedule_at(current_time, attempts)
        periodicity = Setting.update_inventory_devices_periodicity
        time_setting = Setting.update_inventory_devices_time
      
        case periodicity
        when "weekly"
          hour, minute = time_setting.split(":").map(&:to_i)
          current_time.beginning_of_week.advance(weeks: 1, hours: hour, minutes: minute)
        when "daily"
          hour, minute = time_setting.split(":").map(&:to_i)
          current_time.beginning_of_day.advance(days: 1, hours: hour, minutes: minute)
        when "hourly"
          current_time.advance(hours: 1)
        when "every_45_minutes"
          current_time.advance(minutes: 45)
        when "every_30_minutes"
          current_time.advance(minutes: 30)
        when "every_15_minutes"
          current_time.advance(minutes: 15)
        when "every_10_minutes"
          current_time.advance(minutes: 10)
        when "never"
          nil
        else
          # Fallback: try in 12 hours
          current_time.advance(hours: 12)
        end
      end
    end
  end
end
